"use strict";

var _require = require('loader-utils'),
    stringifyRequest = _require.stringifyRequest,
    getOptions = _require.getOptions;

var _require2 = require('path'),
    relative = _require2.relative;

var parseSFCParts = require('./sfc/parser');

var transformScript = require('./transform/script');

var _require3 = require('sfc-compiler'),
    createCompiler = _require3.createCompiler,
    createRenderFn = _require3.createRenderFn,
    baseOptions = _require3.baseOptions;

var modules = require('./modules');

var sfcLoader = require.resolve('.');

var sfcStyleLoader = require.resolve('./sfcStyleLoader');

var stylesheetLoader = require.resolve('stylesheet-loader');

var compiler = createCompiler(Object.assign({}, baseOptions, {
  modules: modules
}));
var builtInRaxModuleName = JSON.stringify('@core/rax');
var builtInRuntimeModuleName = JSON.stringify('@core/runtime');
var runtimeModuleName = JSON.stringify(require.resolve('./helpers/runtime'));
var raxModuleName = JSON.stringify('rax');

module.exports = function (rawContent) {
  var context = this;
  var callback = this.async();
  var contextPath = this.rootContext || this.options && this.options.context || process.cwd();
  var filePath = this.resourcePath;
  var userOptions = getOptions(this) || {};
  var relativePath = relative(contextPath, filePath);

  var _parseSFCParts = parseSFCParts(rawContent),
      template = _parseSFCParts.template,
      script = _parseSFCParts.script,
      styles = _parseSFCParts.styles;
  /**
   * If `part` passed, return SFC's pointed part.
   */


  if (userOptions.part === 'style') {
    return callback(null, styles ? styles.content : '');
  }

  var declarationName = '__sfc_module_declaration__';

  var _transformScript = transformScript(script.content, declarationName, relativePath, rawContent),
      declarationCode = _transformScript.declarationCode,
      sourceMap = _transformScript.sourceMap,
      scopeIdentifiers = _transformScript.scopeIdentifiers;

  var _compiler$compile = compiler.compile(template.content, {
    scopeRefIdentifiers: scopeIdentifiers,

    /**
     * Rax prefer false to make it more similar to JSX,
     * default to false.
     */
    preserveWhitespace: !!userOptions.preserveWhitespace,
    cssInJS: userOptions && userOptions.cssInJS
  }),
      render = _compiler$compile.render,
      ast = _compiler$compile.ast;

  if (!ast) {
    console.warn("Template is empty or not valid, please check " + relative(contextPath, filePath) + "!");
  }

  var renderFn = ast ? createRenderFn(render, {
    loaderContext: context,
    tagHelperMap: ast.tagHelperMap,
    weexGlobalComponents: userOptions.weexGlobalComponents || null,
    stringifyRequest: stringifyRequest
  }) : 'function() { return function() { return ""; } }';
  /**
   * By default, sfc loader will create style tag for CSS text.
   * if RN/Weex style CSS is needed, enable `cssInJS` in loader option.
   */

  var loadStyleRequest;

  if (userOptions.cssInJS === true) {
    var styleLoader = stylesheetLoader + "?disableLog=true&transformDescendantCombinator=true"; // prefix -! means stop any pitching loaders

    loadStyleRequest = stringifyRequest(context, "-!" + styleLoader + "!" + sfcLoader + "?part=style!" + filePath);
  } else {
    loadStyleRequest = stringifyRequest(context, sfcStyleLoader + "!" + filePath + "?style");
  }

  var sfcRuntimeModuleName = userOptions.builtInRuntime ? builtInRuntimeModuleName : runtimeModuleName;
  /**
   * support ESModules / commonjs exportation
   */

  var moduleExportsWrapper = [];

  if (userOptions.module === 'commonjs') {
    // commonjs
    moduleExportsWrapper[0] = 'module.exports =';
    moduleExportsWrapper[1] = '';
  } else {
    // ESModules
    moduleExportsWrapper[0] = 'exports.__esModule = true; exports.default =';
    moduleExportsWrapper[1] = '.default';
  }

  var output = declarationCode + ";\n    " + moduleExportsWrapper[0] + " require(" + sfcRuntimeModuleName + ")" + moduleExportsWrapper[1] + "(\n      typeof " + declarationName + " === 'undefined' ? {} : " + declarationName + ",\n      " + renderFn + ",\n      require(" + loadStyleRequest + "),\n      require(" + (userOptions.builtInRax ? builtInRaxModuleName : raxModuleName) + ")\n    );"; // whether webpack's devtool is configured

  if (this.sourceMap) {
    callback(null, output, sourceMap);
  } else {
    callback(null, output);
  }
};